import javax.swing.* ;
import javax.media.j3d.* ;
import com.sun.j3d.utils.universe.* ;
import javax.vecmath.* ;
import com.sun.j3d.loaders.objectfile.ObjectFile ;
import com.sun.j3d.loaders.* ;
import java.io.FileNotFoundException ;
import com.sun.j3d.utils.behaviors.mouse.* ;
import java.awt.GraphicsConfiguration;

/**
 * Classe principale de l'application.
 * Cette classe est responsable de l'ouverture d'une fenetre de 
 * visualisation, d'une fenetre de controle 
 * et du chargement des differents objets 3D
 */
public class Appli extends JFrame
{
    /**
     * Composant graphique (awt) destine a recevoir l'image generee
     */
    protected Canvas3D canvas ;
    
    /**
     * Noeud transform contenant l'ensemble des objets de la scene 3D
     * Ce noeud permet en particulier de faire pivoter ou zoomer 
     * l'ensemble de la scene directement avec la souris
     */
    protected TransformGroup transformScene ;
    
    /**
     * Univers de l'application. 
     * l'Univers est le container du graphe de scene de l'application.
     * Il contient entre autre les differents objets 3D ainsi que la 
     * ViewPlatfom
     */
    protected SimpleUniverse universe ;
    
    /**
     * Sommet du graphe de scene de description des differents objets 3D
     */
    protected BranchGroup scene ;

    /** 
     * Robot de la simulation
     */
    protected Robot robot ;

    /**
     * Sol de la simulation
     */
    protected Sol sol ;

    /**
     * Reference vers la fenetre principale de l'application
     */
    public static Appli mainFrame ;

    /**
     * Constructeur de la classe
     */
    public Appli() 
    {
	super("Robot") ;
	pack() ;
	setSize(320,240) ;

	mainFrame = this ;


	// fermeture de la fenetre lorsque l'on clique
	// sur le widget correspondant
	setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();

	//canvas = new Canvas3D(getGraphicsConfiguration()) ;
	canvas = new Canvas3D(config) ;
	getContentPane().add("Center", canvas) ;
	
	// creation de l'univers
	universe = new SimpleUniverse(canvas) ;

	// creation de la branche de scene du graphe de scene
	scene = new BranchGroup() ;

	// on donne la possibilite a l'application de lire
	// la sphere englobante de la scene
	scene.setCapability(BranchGroup.ALLOW_BOUNDS_READ) ;


	// mise en place du fond
	Background background = new Background(new Color3f(0.05f,0.05f,0.5f)) ;
	BoundingSphere boundsSphere = 
	    new BoundingSphere(new Point3d(0.0,0.0,0.0), 2000.0);
        background.setApplicationBounds(boundsSphere);
 	scene.addChild(background) ;

	// creation du noeud transform contenant l'ensemble des objets 
	// de la scene
	transformScene = new TransformGroup() ;

	// on ajoute des lumieres
	Color3f light1Color = new Color3f(1.5f, 1.0f, 0.9f);
	Vector3f light1Direction  = new Vector3f(4.0f, -7.0f, -12.0f);
	DirectionalLight light1
	    = new DirectionalLight(light1Color, light1Direction);
	light1.setInfluencingBounds(boundsSphere);
	transformScene.addChild(light1);

	Color3f light2Color = new Color3f(1.5f, 1.0f, 0.9f);
	Vector3f light2Direction  = new Vector3f(4.0f, 7.0f, -12.0f);
	DirectionalLight light2
	    = new DirectionalLight(light2Color, light2Direction);
	light2.setInfluencingBounds(boundsSphere);
	transformScene.addChild(light2);


	// on donne la possibile a l'application (et en particulier
	// a la gestion de la souris) d'aller modifier apres compilation
	// les valeurs de la tranformation 
	transformScene.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	transformScene.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	scene.addChild(transformScene);

	// on associe les mouvements de la souris a des actions
	// sur le noeud transformGroup

	// rotation de la scene sur boutton gauche
	MouseRotate behavior = new MouseRotate();
	behavior.setTransformGroup(transformScene);
	scene.addChild(behavior);
	behavior.setSchedulingBounds(boundsSphere);

	// zoom de la scene sur boutton du milieu
	MouseZoom behavior2 = new MouseZoom();
	behavior2.setTransformGroup(transformScene);
	scene.addChild(behavior2);
	behavior2.setSchedulingBounds(boundsSphere);

	// translation de la scene sur boutton de droite
	MouseTranslate behavior3 = new MouseTranslate();
	behavior3.setTransformGroup(transformScene);
	scene.addChild(behavior3);
	behavior3.setSchedulingBounds(boundsSphere);


	// Creation des differents objets de la scene
	sol = new Sol() ;
	transformScene.addChild(sol.getSol()) ;

	robot = new Robot() ;
	transformScene.addChild(robot.getRobot()) ;

	// ouverture du panneau de controle
	new ControlPanel(robot).show() ;

	scene.compile() ;
	universe.addBranchGraph(scene) ;

	viewAll() ;
    }

    /**
     * Positionne la camera de maniere a pouvoir voir la totalite de la scene
     */
    public void viewAll()
    {
	BoundingSphere mybounds = (BoundingSphere) scene.getBounds() ;
	double t[] = new double[3] ;
	Point3d center=new Point3d() ;
	mybounds.getCenter(center) ;
	center.get(t) ;

	// on positionne la camera de facon a voir la sphere
	TransformGroup cameraPosition = universe.getViewingPlatform().
	    getViewPlatformTransform() ;

	Transform3D trans = new Transform3D() ;
	trans.setTranslation(new Vector3f((float)t[0], 
				  (float)t[1], 
				  (float)t[2]+(float) mybounds.getRadius()*2));
	cameraPosition.setTransform(trans) ;
    }


    /** 
     * Point d'entree de l'application
     */
    static public void main(String[] args)
    {
	new Appli().show() ;
    }
}
